"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const server_1 = require("react-dom/server");
const html_react_parser_1 = require("html-react-parser");
const domelementtype_1 = require("domelementtype");
const html_dom_parser_1 = __importDefault(require("html-dom-parser"));
const css_select_1 = __importDefault(require("css-select"));
const deepmerge_1 = __importDefault(require("deepmerge"));
const renderer_1 = require("@react-pdf/renderer");
const typeMapping = {
    div: renderer_1.View,
    view: renderer_1.View,
    svg: renderer_1.Svg,
    line: renderer_1.Line,
    polyline: renderer_1.Polyline,
    polygon: renderer_1.Polygon,
    path: renderer_1.Path,
    rect: renderer_1.Rect,
    circle: renderer_1.Circle,
    ellipse: renderer_1.Ellipse,
    text: renderer_1.Text,
    tspan: renderer_1.Tspan,
    span: renderer_1.Text,
    title: renderer_1.Text,
    g: renderer_1.G,
    stop: renderer_1.Stop,
    defs: renderer_1.Defs,
    clippath: renderer_1.ClipPath,
    lineargradient: renderer_1.LinearGradient,
    radialgradient: renderer_1.RadialGradient,
    ul: renderer_1.View,
    li: renderer_1.View
};
function convertUnitValue(value, fontSize) {
    const stringValue = value.toString();
    if (stringValue.endsWith('rem')) {
        return Number.parseFloat(stringValue.split('rem')[0]) * fontSize;
    }
    if (stringValue.endsWith('em')) {
        return Number.parseFloat(stringValue.split('em')[0]) * fontSize;
    }
    return value;
}
function SvgWrapper(props) {
    const { rechartResponsive = true, baseFontSize = 11, propsUpdateQuery: elementProps = {}, styleUpdateQuery: elementStyles = {} } = props;
    let html = (0, server_1.renderToStaticMarkup)(props.children);
    const dom = (0, html_dom_parser_1.default)(html, { lowerCaseAttributeNames: false });
    const defaultElementStyles = {
        'tspan, text, span': {
            fontSize: baseFontSize
        },
        '.recharts-legend-wrapper': {
            position: undefined,
            width: undefined,
            flexDirection: 'row',
            justifyContent: 'center',
            alignItems: 'center'
        },
        '.recharts-default-legend': {
            flexDirection: 'row',
            justifyContent: 'center',
            flexWrap: 'wrap',
            gap: 8,
        },
        '.recharts-legend-item': {
            flexDirection: 'row',
            gap: 4
        },
    };
    if (rechartResponsive) {
        defaultElementStyles['.recharts-wrapper, .recharts-wrapper > svg'] = {
            width: '100%',
            height: 'auto'
        };
    }
    const updatePropsQueries = Object.entries(elementProps).concat(Object.entries(defaultElementStyles)
        .concat(Object.entries(elementStyles))
        .map(([selector, style]) => ([selector, { style }])));
    const nodeAdditionalProps = updatePropsQueries.flatMap(([selector, newProps]) => (0, css_select_1.default)(selector, dom).map(node => ({
        node,
        additionalPropsGetter: (currentProps) => {
            if (typeof newProps === 'function') {
                return newProps(currentProps);
            }
            return newProps;
        }
    })));
    const parserOptions = {
        transform(reactNode, domNode, index) {
            var _a, _b;
            switch (domNode.type) {
                case domelementtype_1.ElementType.Text:
                    return domNode.data;
                case domelementtype_1.ElementType.Tag:
                    const elementType = typeMapping[domNode.name];
                    if (!elementType)
                        return null;
                    let { children, ...props } = reactNode.props;
                    props.key = props.key || reactNode.key || index;
                    const fontSize = ((_a = props.style) === null || _a === void 0 ? void 0 : _a.fontSize) || baseFontSize;
                    if (props.style) {
                        Object.entries(props.style)
                            .filter(([prop, value]) => value != null)
                            .forEach(([prop, value]) => {
                            props.style[prop] = convertUnitValue(value, fontSize);
                        });
                    }
                    if (props.strokeDasharray) {
                        props.strokeDasharray = props.strokeDasharray.split(' ').join(', ');
                    }
                    if (domNode.name == 'tspan') {
                        props.dx = convertUnitValue(props.dx || 0, fontSize);
                        props.dy = convertUnitValue(props.dy || 0, fontSize);
                    }
                    if (domNode.name == 'text') {
                        children = react_1.default.Children.map(children, child => {
                            if ((child === null || child === void 0 ? void 0 : child.type) != 'TSPAN')
                                return child;
                            const coordinates = {};
                            if (child.props.dx) {
                                coordinates.x = child.props.dx + Number.parseFloat(props.x);
                            }
                            if (child.props.dy) {
                                coordinates.y = child.props.dy + Number.parseFloat(props.y);
                            }
                            if (coordinates.x || coordinates.y) {
                                return react_1.default.cloneElement(child, coordinates);
                            }
                            return child;
                        });
                        if ((_b = props.transform) === null || _b === void 0 ? void 0 : _b.startsWith('rotate(')) {
                            const mathes = props.transform.matchAll(new RegExp('[-.0-9]+', 'g'));
                            const [rotate, posX = props.x, posY = props.y] = Array.from(mathes).map((value) => value[0]);
                            if (!props.style)
                                props.style = {};
                            props.style.transformOrigin = `${posX} ${posY}`;
                        }
                    }
                    nodeAdditionalProps.filter(x => x.node == domNode).forEach(({ additionalPropsGetter }) => {
                        props = (0, deepmerge_1.default)(props, additionalPropsGetter(props));
                    });
                    return react_1.default.createElement(elementType, props, children);
                default:
                    return null;
            }
        }
    };
    return (0, html_react_parser_1.domToReact)(dom, parserOptions);
}
exports.default = SvgWrapper;
